/********************************************************AUTOKORREL*************************************************************/

/* 	Programm zur Berechnung der Autokorrelationskoeffizienten einer Messwert-Zeitreihe, im vorliegenden der Geschwindigkeits-
	messwerte und des turbulenten Zeitmassstabes der drei Geschwindigkeitskomponenten. Die Autokortrelationsfunktion
	entspricht grundlegend der Definitionm von Nielsen und Wendroth (2003). Somit besteht Unabhängigkeit von der Stationarität 
	des Prozesses.
	Eingabedateien: Messpunktdateien mit dem Format: Zeit y- z-Koordinate u- v- w-Komponente der Geschwindigkeit
	Die Ausgabedatei Zeitmass.tz beinhaltet das Format: y- z-Koordinate integraler Zeitmaßstab in u- v- w-Richtung         */

#include<stdio.h>
#include<string.h>
#include<math.h>
#include<stdlib.h>

#define MAX_LAENGE 50000
#define PFAD_LAENGE 255

typedef struct feld{
	double r[3][MAX_LAENGE/2];
} feld;

/* Unterfunktion zur Ermittlung der Autokorrelationskoeffizienten fuer die Geschwindigkeitsfluktuationen in x, y und z-Richtung. Eingabeparameter ist die koordinatentransformierte Punktdatei mit t y z u v w - Format. Ausgabe ist das mehrdimensionale Feld der Autokorrelationskoeffizienten. Voraussetzung ist die math.h-Bibo */

feld autokorrel (FILE *datei){

	int i=1, tau, n=0, weiter;
	double u[MAX_LAENGE], v[MAX_LAENGE], w[MAX_LAENGE], Sum_u[MAX_LAENGE], Sum_v[MAX_LAENGE], Sum_w[MAX_LAENGE], u_mittel, v_mittel, w_mittel;
	double u_mittelplus, v_mittelplus, w_mittelplus, SumSqDiff_u=0, SumSqDiff_v=0, SumSqDiff_w=0;
	double SumSqDiff_uplus=0, SumSqDiff_vplus=0, SumSqDiff_wplus=0, ProdDiff_u=0, ProdDiff_v=0, ProdDiff_w=0;
	double Stabw_u, Stabw_v, Stabw_w, Stabw_uplus, Stabw_vplus, Stabw_wplus, Cov_u, Cov_v, Cov_w;
	double t, y, z;
	char zeile[200];
	feld ans;
	
	Sum_u[0]=0;
	Sum_v[0]=0;
	Sum_w[0]=0;
	
	while(!feof(datei)){				/* Zaehlen, Einlesen und Summenbildung der Messwerte */
	
		fgets(zeile,200,datei);
		weiter=feof(datei);
		if(weiter==0){
			sscanf(zeile,"%lf%lf%lf%lf%lf%lf",&t,&y,&z,&u[i],&v[i],&w[i]);
			Sum_u[i]=Sum_u[i-1]+u[i];	
			Sum_v[i]=Sum_v[i-1]+v[i];
			Sum_w[i]=Sum_w[i-1]+w[i];
			i++;
			n++;
		}
	}
	
	fseek(datei,0L,SEEK_SET);				/* Zuruecksetzen an Anfang der Datei */
	
	for(tau=0;tau<n/2;tau++){				/* Schleife je Zeitschritt tau: Mittelwert, Stdabw, Cov und r(tau) */
		
		u_mittel=Sum_u[n-tau]/(n-tau);			//Mittlere Stroemungsgeschwindigkeiten (von Beginn der Datenreihe aus)
		v_mittel=Sum_v[n-tau]/(n-tau);
		w_mittel=Sum_w[n-tau]/(n-tau);
		
		u_mittelplus=(Sum_u[n]-Sum_u[tau])/(n-tau);	//Mittlere Stroemungsgeschwindigkeiten (vom Ende der Datenreihe aus)
		v_mittelplus=(Sum_v[n]-Sum_v[tau])/(n-tau);
		w_mittelplus=(Sum_w[n]-Sum_w[tau])/(n-tau);
		
		for(i=1;i<=n-tau;i++){				/* Schleife fuer Summen quadrierter Abweichungen */
			
			SumSqDiff_u+=(u[i]-u_mittel)*(u[i]-u_mittel);		//(Beginn)
			SumSqDiff_v+=(v[i]-v_mittel)*(v[i]-v_mittel);
			SumSqDiff_w+=(w[i]-w_mittel)*(w[i]-w_mittel);
			
			SumSqDiff_uplus+=(u[i+tau]-u_mittelplus)*(u[i+tau]-u_mittelplus);	//(Ende)
			SumSqDiff_vplus+=(v[i+tau]-v_mittelplus)*(v[i+tau]-v_mittelplus);
			SumSqDiff_wplus+=(w[i+tau]-w_mittelplus)*(w[i+tau]-w_mittelplus);
			
			ProdDiff_u+=(u[i]-u_mittel)*(u[i+tau]-u_mittelplus);
			ProdDiff_v+=(v[i]-v_mittel)*(v[i+tau]-v_mittelplus);
			ProdDiff_w+=(w[i]-w_mittel)*(w[i+tau]-w_mittelplus);
			
		}
		
		Stabw_u=sqrt(SumSqDiff_u/(n-tau));			//Standardabweichung (Beginn)
		Stabw_v=sqrt(SumSqDiff_v/(n-tau));
		Stabw_w=sqrt(SumSqDiff_w/(n-tau));
		
		Stabw_uplus=sqrt(SumSqDiff_uplus/(n-tau));		//Standardabweichung (Ende)
		Stabw_vplus=sqrt(SumSqDiff_vplus/(n-tau));
		Stabw_wplus=sqrt(SumSqDiff_wplus/(n-tau));
		
		Cov_u=ProdDiff_u/(n-tau);				//Kovarianz(Tau)
		Cov_v=ProdDiff_v/(n-tau);
		Cov_w=ProdDiff_w/(n-tau);
		
		ans.r[0][tau]=Cov_u/(Stabw_u*Stabw_uplus);		//Speicherung der Autokorrelationskoeffizienten(Tau)
		ans.r[1][tau]=Cov_v/(Stabw_v*Stabw_vplus);
		ans.r[2][tau]=Cov_w/(Stabw_w*Stabw_wplus);
		
		ans.r[0][n/2]='\0';					//Schutz gegen Ueberschreiben
		ans.r[1][n/2]='\0';
		ans.r[2][n/2]='\0';
		
	}
	return ans;
}

/* Unterprogramm zur Berechnung der Flaeche unterhalb der Autokorrelationsfunktion nach Trapezregel mit Wahlpflicht der Fluktuationskomponente u, v oder w */

double integral_autokorrel(char Komponente, feld korrelkoeff){
	
	int i=0, k;
	double Sum_IntAut=0;
	
	switch(Komponente){		//Auswahl der Geschwindigkeitskomponente, bzw. Richtung
		case'u':
			k=0;
			break;
		case'v':
			k=1;
			break;
		case'w':
			k=2;
			break;
	}
	
	do{
		Sum_IntAut+=(korrelkoeff.r[k][i]+korrelkoeff.r[k][i+1])/2;	//Integral Korrelationskoeffizienten ueber konst.Zeitschrittweite=1
		i++;
	}while(korrelkoeff.r[k][i+1]);
	
	return Sum_IntAut;
}

/* Hauptprogramm zur Dateiverarbeitung und Erstellung. Grundlage sind die koordinatentransformierten Punktdateien */
/* Eingebunden in dem Projekt ADDV, welches im Callback die Argumente uebergibt! */

main(int argc, char *argv[]){
	
	
	FILE *eingabe, *ausgabe, *fkt_ausg;
	char pfad[PFAD_LAENGE], zielpfad[PFAD_LAENGE], y_pfad[PFAD_LAENGE], y_zielpfad[PFAD_LAENGE], zy_pfad[PFAD_LAENGE];
	char zy_zielpfad[PFAD_LAENGE];
	char lamellenordner[5], lamellenpkt[5], punkt[5], tp_datei[PFAD_LAENGE], tz_datei[PFAD_LAENGE];
	char rt_datei[PFAD_LAENGE];
	char tp_zeile[150];
	double t, y, z, u_muell, v_muell, w_muell, dt;
	double int_u, int_v, int_w, t_u, t_v, t_w, rt_u, rt_v, rt_w;
	int i,j,k,l,n=9,m=25, p, haken;
	feld ergebnis;
	
	sprintf(pfad,"%s",argv[1]);						/* Einlesen des Ordners */
	
	sprintf(zielpfad,"%s",argv[2]);			/* Einlesen des Zielpfades */
	
	haken=atoi(argv[3]);
	
	strcpy(tz_datei,zielpfad);
	strcat(tz_datei,".tz");
	
	ausgabe=fopen(tz_datei,"w");
	
	
	for(i=0;i<=m;i++){						/* Schleife fuer 1. Lamellenwert */
		for(j=0;j<=n;j++){					/* Schleife fuer 2. Lamellenwert */
		for(k=0;k<=n;k++){					/* Schleife fuer 3. Lamellenwert (Zweite Nachkommastelle) */
		
			strcpy(y_pfad,pfad);				/* Kopieren und Ueberschreiben des y-zielpfad mit zielpfad */
			strcpy(y_zielpfad,zielpfad);
			sprintf(lamellenordner,"/%i_%i%i",i,j,k);
			strcat(y_pfad,lamellenordner);			/* Lamellenordner an Pfad anhaengen */
			strcat(y_zielpfad,lamellenordner);
			
			for(l=1;l<=m;l++){				/* Schleife fuer Lamellenpunkt */ 
			
				strcpy(zy_pfad,y_pfad);			/* Kopieren bzw. Ueberschreiben des zypfad mit ypfad */
				strcpy(zy_zielpfad,y_zielpfad);
				sprintf(lamellenpkt,"/%i",l);
				strcat(zy_pfad,lamellenpkt);		/* Messpunkt an Pfad anhaengen */
				sprintf(punkt,"_%i",l);
				strcat(zy_zielpfad,punkt);

				strcpy(tp_datei,zy_pfad);
				strcpy(rt_datei,zy_zielpfad);
				strcat(tp_datei,".tp");
				strcat(rt_datei,".rt");
			
				eingabe=fopen(tp_datei,"r");		/* Oeffnen der Punktdatei */
				
				if(eingabe!=NULL){
					
					fgets(tp_zeile,150,eingabe);
					sscanf(tp_zeile,"%lf%lf%lf%lf%lf%lf",&t,&y,&z,&u_muell,&v_muell,&w_muell);
					
					fseek(eingabe,0L,SEEK_SET);
					
					ergebnis=autokorrel(eingabe);		//Starten des Unterprogramms zur Berechnung der Korrelationskoeffizienten
					
					int_u=integral_autokorrel('u',ergebnis);	//Integration der Koeffizienten zur integralen Zeitskale, Uebergabe 
					int_v=integral_autokorrel('v',ergebnis);	//jeweils durch Speicherung im feld: name.r[][]
					int_w=integral_autokorrel('w',ergebnis);
					
					t_u=int_u*0.04;
					t_v=int_v*0.04;
					t_w=int_w*0.04;
					
					fprintf(ausgabe,"%lf\t%lf\t%lf\t%lf\t%lf\n",y,z,t_u,t_v,t_w);
					
					if(haken==1)
					fkt_ausg=fopen(rt_datei,"w");		/* Schreiben der Korrellationskoeff je Punkt */
					
					dt=0.02;
					p=0;
					
					while(ergebnis.r[0][p]){		//Verlauf der Autokorrelationsfunktion
					
						rt_u=ergebnis.r[0][p];
						rt_v=ergebnis.r[1][p];
						rt_w=ergebnis.r[2][p];
						if(haken==1)
						fprintf(fkt_ausg,"%lf\t%lf\t%lf\t%lf\n",dt,rt_u,rt_v,rt_w);
						p++;
						dt+=0.04;
						
					}
					if(haken==1) fclose(fkt_ausg);
					fclose(eingabe);
				}
			}
		}
		}
	}
	fclose(ausgabe);
	printf("Fertig\n");
}
