/*********************************************************TAU_SEKUND*******************************************************************/

/* 
	Programm zur Berechnung der Verteilung der Impulsübertragung in Groessenordnung einer Schubspannung im Profilquerschnitt. 
	Format der Ausgabedatei: 	y-  z-Koordinate  Gesamtschubspannung  Anteil durch Sekundaerstroemung  durch Turbulenz
					Wirkungsrichtung (Winkel gegen die Horizontale) der Gesamt.  Sekund.  turbulenten Schubsp.    */

#include<stdio.h>
#include<string.h>
#include<math.h>
#include<stdlib.h>

#define PFAD_LAENGE 255
#define ZEILEN_LAENGE 250
#define PI 3.14159265


// Berechnungsalgorithmen und Felddefinition komplexer Zahlen aus Press et.al. (1994)

typedef struct FCOMPLEX{double r,i;}fcomplex;

// Addition komplexer Zahlen

fcomplex Cadd(fcomplex a, fcomplex b){
	fcomplex c;
	c.r=a.r+b.r;
	c.i=a.i+b.i;
	return c;
}

// Subtraktion komplexer Zahlen

fcomplex Csub(fcomplex a, fcomplex b)
{
	fcomplex c;
	c.r=a.r-b.r;
	c.i=a.i-b.i;
	return c;
}

// Multiplikation komplexer Zahlen

fcomplex Cmul(fcomplex a, fcomplex b)
{
	fcomplex c;
	c.r=a.r*b.r-a.i*b.i;
	c.i=a.i*b.r+a.r*b.i;
	return c;
}

// Absoluter Wert der komplexen Zahl

double Cabs(fcomplex z)
{
	float x,y,ans,temp;
	x=fabs(z.r);
	y=fabs(z.i);
	if (x == 0.0)
		ans=y;
	else if (y == 0.0)
		ans=x;
	else if (x > y) {
		temp=y/x;
		ans=x*sqrt(1.0+temp*temp);
	} else {
		temp=x/y;
		ans=y*sqrt(1.0+temp*temp);
	}
	return ans;
}

/*	Hauptprogramm mit Uebergabeparametern aus dem Projekt ADDV		*/

main(int argc, char *argv[]){
	
	
	FILE *eingabe1, *eingabe2, *ausgabe;
	char pfad[PFAD_LAENGE], zielpfad[PFAD_LAENGE], y_pfad[PFAD_LAENGE],zy_pfad[PFAD_LAENGE];
	char lamellenordner[5], lamellenpkt[5], tp_datei[PFAD_LAENGE], txt_datei[PFAD_LAENGE], kot_datei[PFAD_LAENGE];
	char tp_zeile[ZEILEN_LAENGE], kot_zeile[ZEILEN_LAENGE];
	double t, y, z,  v, w, vm, wm,tau_ges, tau_sekundaer, tau_flukt, rho=1000.0, u_dummy,phi,phi_m,phi_s;
	fcomplex um, u, nm, n, ns, us, Prod_u_n, Prod_um_nm, Prod_us_ns;
	int i,j,k,l,o=9,m=25, weiter, anzahl;
	
	sprintf(pfad,"%s",argv[1]);				/* Einlesen des Ordners */
	
	sprintf(zielpfad,"%s",argv[2]);				/* Einlesen des Zielpfades */
	
	strcpy(txt_datei,zielpfad);
	strcat(txt_datei,".sek");
	
	ausgabe=fopen(txt_datei,"w");
	
	strcpy(kot_datei,pfad);		
	strcat(kot_datei,"/Punktmittel.kot");
	
	eingabe1=fopen(kot_datei,"r");				// Oeffnen der Mittelwert-Datei
		
	for(i=0;i<=m;i++){						/* Schleife fuer 1. Lamellenwert */
	
		for(j=0;j<=o;j++){					/* Schleife fuer 2. Lamellenwert */
		for(k=0;k<=o;k++){					/* Schleife fuer 3. Lamellenwert (Zweite Nachkommastelle) */
		
			strcpy(y_pfad,pfad);				/* Kopieren und Ueberschreiben des y-zielpfad mit zielpfad */
			sprintf(lamellenordner,"/%i_%i%i",i,j,k);
			strcat(y_pfad,lamellenordner);			/* Lamellenordner an Pfad anhaengen */
			
			
			for(l=1;l<=m;l++){				/* Schleife fuer Lamellenpunkt */ 
			
				strcpy(zy_pfad,y_pfad);			/* Kopieren bzw. Ueberschreiben des zypfad mit ypfad */
				sprintf(lamellenpkt,"/%i",l);
				strcat(zy_pfad,lamellenpkt);		/* Messpunkt an Pfad anhaengen */
				strcpy(tp_datei,zy_pfad);
				strcat(tp_datei,".tp");
			
				eingabe2=fopen(tp_datei,"r");		/* Oeffnen der Punktgeschwindigkeits-Datei */
				
				if(eingabe2!=NULL){
					
					anzahl=0;			// Startwerte (0) aller Zaehler und Produkte
					Prod_um_nm.r=0.0;
					Prod_um_nm.i=0.0;
					Prod_us_ns.r=0.0;
					Prod_us_ns.i=0.0;
					Prod_u_n.r=0.0;
					Prod_u_n.i=0.0;
					
					fgets(kot_zeile,250,eingabe1);
					sscanf(kot_zeile,"%lf%lf%lf%lf%lf",&y,&z,&u_dummy,&vm,&wm);	//Einlesen der Mittelwerte
					um.r=u_dummy;							// Mittelwerte als komplexe Zahlen
					um.i=0.0;
					
					nm.r=vm;
					nm.i=wm;
					
					Prod_um_nm=Cmul(um,nm);						// Produkt der beiden Mittelwerte
					
					while(!feof(eingabe2)){
						fgets(tp_zeile,250,eingabe2);
						weiter=feof(eingabe2);
						if(weiter==0){
						
							sscanf(tp_zeile,"%lf%lf%lf%lf%lf%lf",&t,&y,&z,&u_dummy,&v,&w);	// Einlesen der Punktgeschwindigkeiten (momentan)
							u.r=u_dummy;					// Momentangeschwindigkeiten als komplexe Zahlen
							u.i=0.0;
							
							n.r=v;
							n.i=w;
							
							ns=Csub(n,nm);					// Berechnung der (komplexen) Fluktuationen
							us=Csub(u,um);
							
							Prod_us_ns=Cadd(Prod_us_ns,Cmul(us,ns));	// Kovarianz der Fluktuationen
							Prod_u_n=Cadd(Prod_u_n,Cmul(u,n));		// Kovarianz der Mittelwerte
							
							anzahl++;
							}
					}
					
					tau_sekundaer=rho*Cabs(Prod_um_nm)/10000.0;			//Impulstransport (Schubspannung) durch Sekundaerstroemung
					tau_ges=rho*Cabs(Prod_u_n)/(anzahl*10000.0);			//Impulstransport (Schubspannung) insgesamt
					tau_flukt=rho*Cabs(Prod_us_ns)/(anzahl*10000.0);		//Impulstransport (Schubspannung) durch Fluktuationen (Turbulenz)
					
					if(Prod_um_nm.r>0.0)	phi_m=atan2(Prod_um_nm.i,Prod_um_nm.r);		//Winkel (Wirkungsrichtung) - Sekundarstroemung
					else if(Prod_um_nm.i<0.0) phi_m=atan2(Prod_um_nm.i,Prod_um_nm.r)-PI;
					else phi_m=atan2(Prod_um_nm.i,Prod_um_nm.r)+PI;
					
					if(Prod_u_n.r>0.0)	phi=atan2(Prod_u_n.i,Prod_u_n.r);		//Winkel (Wirkungsrichtung) - Gesamt
					else if(Prod_u_n.i<0.0) phi=atan2(Prod_u_n.i,Prod_u_n.r)-PI;
					else phi=atan2(Prod_u_n.i,Prod_u_n.r)+PI;
					
					if(Prod_us_ns.r>0.0)	phi_s=atan2(Prod_us_ns.i,Prod_us_ns.r);		//Winkel (Wirkungsrichtung) - Fluktuationen
					else if(Prod_us_ns.i<0.0) phi_s=atan2(Prod_us_ns.i,Prod_us_ns.r)-PI;
					else phi_s=atan2(Prod_us_ns.i,Prod_us_ns.r)+PI;
					
					
					fclose(eingabe2);
fprintf(ausgabe,"%lf\t%lf\t%lf\t%lf\t%lf\t%lf\t%lf\t%lf\n",y,z,tau_ges,tau_sekundaer,tau_flukt,phi,phi_m,phi_s);		//Schreiben der Ausgabe
//fprintf(ausgabe,"%lf\t%lf\t%lf\t%lf\t%lf\n",y,z,Prod_u_n.r/(10000.0*anzahl),Prod_um_nm.r/(10000.0),Prod_us_ns.r/(10000.0*anzahl));
//fprintf(ausgabe,"%lf\t%lf\t%lf\t%lf\t%lf\n",y,z,Prod_u_n.i/(10000.0*anzahl),Prod_um_nm.i/(10000.0),Prod_us_ns.i/(10000.0*anzahl));
				}
			}
		}
		}
	}
	fclose(ausgabe);
	fclose(eingabe1);
}
