/************************************AUTOPST*******************************************************************/
/*	Neueste Version der Phase-Space-Threshold Methode: Gegenueber den Versionen PST, SonPST erfolgen hier 
	fuenf automatische Durchlaufe des modifizierten PST-Algorithmus bevor die Luecken durch die Spline-Inter-
	polation geschlossen werden. Des Weiteren fuehren zu viele (>10%), mehr als eine mittelgroße Luecke zum 
	Abbruch. Bei großen Lueckenweite (>30) wird nur der laengere Datenreihenteil weiter verarbeitet. Dateien,
	die diesen Kriterien nicht entsprechen, werden geloescht!						*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <stddef.h>
#include "nrutil.h"

//Grenzwerte für CORR und SNR!!!:

#define CORR 60.0
#define SNR 12.0


#define MAX_DATA 30000
#define PFAD 256
#define NR_END 1
#define FREE_ARG char*

/*	Unterprogramme zur Spline-Interpolation, aus Press et.al., 1994		*/

void free_vector(float *v, long nl, long nh)
/* free a float vector allocated with vector() */
{
	free((FREE_ARG) (v+nl-NR_END));
}


float *vector(long nl, long nh)
/* allocate a float vector with subscript range v[nl..nh] */
{
	float *v;

	v=(float *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(float)));
	if (!v) nrerror("allocation failure in vector()");
	return v-nl+NR_END;
}

void nrerror(char error_text[])
/* Numerical Recipes standard error handler */
{
	fprintf(stderr,"Numerical Recipes run-time error...\n");
	fprintf(stderr,"%s\n",error_text);
	fprintf(stderr,"...now exiting to system...\n");
	exit(1);
}

//Berechnung der 2.Abl an i ueber 2 Randbedingungen und N-2 Gleichungen der ersten Ableitungen an x = xi aus [xi-1,xi] und [xi,xi-1]

void spline(float x[], float y[], int n, float yp1, float ypn, float y2[])
{
	int i,k;
	float p,qn,sig,un,*u;

	u=vector(1,n-1);
	if (yp1 > 0.99e30)
		y2[1]=u[1]=0.0;
	else {
		y2[1] = -0.5;
		u[1]=(3.0/(x[2]-x[1]))*((y[2]-y[1])/(x[2]-x[1])-yp1);
	}
	for (i=2;i<=n-1;i++) {
		sig=(x[i]-x[i-1])/(x[i+1]-x[i-1]);
		p=sig*y2[i-1]+2.0;
		y2[i]=(sig-1.0)/p;
		u[i]=(y[i+1]-y[i])/(x[i+1]-x[i]) - (y[i]-y[i-1])/(x[i]-x[i-1]);
		u[i]=(6.0*u[i]/(x[i+1]-x[i-1])-sig*u[i-1])/p;
	}
	if (ypn > 0.99e30)
		qn=un=0.0;
	else {
		qn=0.5;
		un=(3.0/(x[n]-x[n-1]))*(ypn-(y[n]-y[n-1])/(x[n]-x[n-1]));
	}
	y2[n]=(un-qn*u[n-1])/(qn*y2[n-1]+1.0);
	for (k=n-1;k>=1;k--)
		y2[k]=y2[k]*y2[k+1]+u[k];
	free_vector(u,1,n-1);
}

//Berechnung des durch die kubische Spline interpolierten Wertes y(x)
float splint(float xa[], float ya[], float y2a[], int n, float x)
{
	void nrerror(char error_text[]);
	int klo, khi, k;
	float h, b, a, y;
	
	klo=1;
	khi=n;
	while(khi-klo>1){			//Bestimmung der Stellen ober- und unterhalb des zu interpolierenden Wertes
		k=(khi+klo) >> 1;
		if(xa[k]>x) 
			khi=k;
		else
			klo=k;
	}
	
	h=xa[khi]-xa[klo];			//Differenz zwischen den umgebenden Stuetzstellen
	if(h==0.0) 
		nrerror("Differenz zwischen den Stuetzstellen muss > 0 sein!");
	a=(xa[khi]-x)/h;			//entsprechen A und B bzw. im Weiteren C und D
	b=(x-xa[klo])/h;
	y=a*ya[klo]+b*ya[khi]+((a*a*a-a)*y2a[klo]+(b*b*b-b)*y2a[khi])*(h*h)/6.0;	//Gl. des kub. Splines
	return y;
}

//	Hauptprogramm in dem erst die Luecken identifiziert und anschliessend interpoliert werden (Ersten und letzten beiden Werte duerfen keine Ausreisser sein!)

main(){
	
	FILE *eingabe, *eingabe2, *eingabe3, *ausgabe, *ausgabe2;
	
	int punkt, i, j, k, l, n, n_l, n_v, n_dv, n_d2v, l_anfang, l_ende, weite, imuell,adv;
	int weiter,komponente,fehler,schleife,count,start,stopp,l_nr,stellen, spike, l_m, l_dm, l_cm;
	char pst_datei[PFAD], pfad[PFAD],lamellenpfad[PFAD],punktpfad[PFAD],cor_pfad[PFAD],snr_pfad[PFAD];
	char cor_zeile[200],snr_zeile[200],zeile[200];
	char lamelle[10], lamellenpkt[10],pst_endung[10], cor_pkt[10],snr_pkt[10],cmuell[10];
	double S_v, S_dv, S_d2v, M_v, M_dv, M_d2v, S_vl, S_dvl, S_d2vl, Q_u, Q_du, Q_d2u, Q_v, Q_dv, Q_d2v, S_ud2u, u_teta, d2u_teta;
	double Sigma_u, Sigma_du, Sigma_d2u, lambda_u, lambda_du, lambda_d2u, max_u, max_du, max_d2u, max_v, max_dv, max_d2v;
	double  corx[MAX_DATA],cory[MAX_DATA],corz[MAX_DATA],teta, a, b, Krit1, Krit2, Krit3, snrx[MAX_DATA],snry[MAX_DATA],snrz[MAX_DATA];
	double v[MAX_DATA],vx[MAX_DATA],vy[MAX_DATA],vz[MAX_DATA],t,lf_muell;
	double dv[MAX_DATA],d2v[MAX_DATA],u[MAX_DATA],du[MAX_DATA],d2u[MAX_DATA], v_luecke[MAX_DATA],linear;
	float y2[MAX_DATA], x[MAX_DATA], y[MAX_DATA], vi, a_1, a_n;
	
				//Initialwerte fuer Spike-Schleife und Bilderspeicherplatz
	
	printf("Pfad des Messdatums:\n");
	scanf("%s",&pfad);
	printf("Nortek (0) oder Sontek (1) -ADV?\n");
	scanf("%i",&adv);
	strcpy(pst_datei,pfad);
	sprintf(pst_endung,"/PST.txt");
	strcat(pst_datei,pst_endung);
	ausgabe2=fopen(pst_datei,"w");

for(l_m=0;l_m<=25;l_m++){		//Schleifen zum Oeffnen der Lamellenordner
for(l_dm=0;l_dm<=9;l_dm++){
for(l_cm=0;l_cm<=9;l_cm++){

	sprintf(lamelle,"/%i_%i%i/",l_m,l_dm,l_cm);
	strcpy(lamellenpfad,pfad);
	strcat(lamellenpfad,lamelle);

for(punkt=1;punkt<=20;punkt++){		//Schleife zum Oeffnen der Punktdateien
	
	if(adv==1) sprintf(lamellenpkt,"%i_Probe_1_(A820).vel",punkt);		//Bei SonTek-ADV
	else{ 
		sprintf(lamellenpkt,"%i.vel",punkt);				//Bei Nortek-ADV
		sprintf(cor_pkt,"%i.cor",punkt);
		sprintf(snr_pkt,"%i.snr",punkt);
	}

	strcpy(punktpfad,lamellenpfad);
	strcat(punktpfad,lamellenpkt);

for(komponente=1;komponente<=3;komponente++){			//Schleife zum Durchlaufen der drei Geschwindigkeitskomponenten

	spike=0;
	eingabe=fopen(punktpfad,"r");		//Oeffnen der SonTek-Datei und der Nortek-vel-Datei
	
	if(eingabe==NULL) break;
else{
	
	if(adv==0){						//Oeffnen der Nortek-CORR- und -SNR-Dateien
		strcpy(cor_pfad,lamellenpfad);
		strcat(cor_pfad,cor_pkt);
		eingabe2=fopen(cor_pfad,"r");
		fgets(cor_zeile,200,eingabe2);
		strcpy(snr_pfad,lamellenpfad);
		strcat(snr_pfad,cor_pkt);
		eingabe3=fopen(snr_pfad,"r");
		fgets(snr_zeile,200,eingabe3);
	}

/*+++++++++++++++++Einlesen der Roh-.vel, CORR, SNR-Datei+++++++++++++++++++++++++++++++*/

	fgets(zeile,200,eingabe);
	
	if(adv==1){
	if(sscanf(zeile,"%lf%lf%lf%lf%lf%lf%lf%lf%lf",&vx[0],&vy[0],&vz[0],&corx[0],&cory[0],&corz[0],&snrx[0],&snry[0],&snrz[0])==9) fseek(eingabe,0L,SEEK_SET);
	}
	else{
	if(sscanf(zeile,"%lf%lf%lf%lf%lf%lf",&lf_muell,&lf_muell,&lf_muell,&vx[0],&vy[0],&vz[0])==6) fseek(eingabe,0L,SEEK_SET);
	}

	i=1; n=0;
	while(!feof(eingabe)){
		fgets(zeile,200,eingabe);

		if(adv==0){
			fgets(cor_zeile,200,eingabe2);
			fgets(snr_zeile,200,eingabe3);
		}
		
		weiter=feof(eingabe);
		if(weiter==0){
			
		if(adv==1) sscanf(zeile,"%lf%lf%lf%lf%lf%lf%lf%lf%lf",&vx[i],&vy[i],&vz[i],&corx[i],&cory[i],&corz[i],&snrx[i],&snry[i],&snrz[i]);
		else{ 	sscanf(zeile,"%lf%lf%lf%lf%lf%lf",&lf_muell,&lf_muell,&lf_muell,&vx[i],&vy[i],&vz[i]);
			sscanf(cor_zeile,"%lf%lf%lf%lf%lf%lf",&lf_muell,&lf_muell,&lf_muell,&corx[i],&cory[i],&corz[i]);
			sscanf(snr_zeile,"%lf%lf%lf%lf%lf%lf",&lf_muell,&lf_muell,&lf_muell,&snrx[i],&snry[i],&snrz[i]);
		}

			switch(komponente){	//Komponenten-Auswahl
				case 1:
					v[i]=vx[i];
					break;
				case 2:
					v[i]=vy[i];
					break;
				case 3:
					v[i]=vz[i];
					break;
				}
			i++;
			n++;
		}
	}
	fclose(eingabe);

	if(adv==0){
		fclose(eingabe2);
		fclose(eingabe3);
	}
	
	S_v=0.0; S_dv=0.0; S_d2v=0.0;		//Startwerte der Summen
	for(i=1;i<=n;i++){
		S_v+=v[i];
		if(i>1 && i<n){
			dv[i]=(v[i+1]-v[i-1])/2;
			S_dv+=dv[i];
		}
		if(i>2 && i<n-1){
			d2v[i]=(dv[i+1]-dv[i-1])/2;
			S_d2v+=d2v[i];
		}
	}
	M_v=S_v/n;			//Mittelwerte der Geschwindigkeit, Steigung, Kruemmung
	M_dv=S_dv/(n-2);
	M_d2v=S_d2v/(n-4);
	
/***Schleife, wenn optische Kontrolle dazu fuehrt, dass weitere Ersetzungen /Luecken folgen sollen***/
	
//	schleife=0;										//Initialwerte fuer Schleifenzaehler
	
	count=0;
	while(spike<5){				// 5 maliges Durchlaufen der PST-Schleife
			
		S_vl=0.0; S_dvl=0.0; S_d2vl=0.0; n_l=0;
		for(i=3;i<=n-2;i++){							//Summenbildung der Ausreisserwerte
			if(v[i]==v_luecke[i]){
				S_vl+=v[i];						//fuer Geschwindigkeit
				S_dvl+=dv[i];						//fuer Steigung
				S_d2vl+=d2v[i];						//fuer Kruemmung
				n_l++;
			}
		}
		n_v=n-n_l;
		n_dv=n-n_l-2;
		n_d2v=n-n_l-4;
		M_v=(S_v-S_vl)/n_v;							//Mittelwerte nach Bereinigung der Datenreihe von Ausreissern
		M_dv=(S_dv-S_dvl)/n_dv;
		M_d2v=(S_d2v-S_d2vl)/n_d2v;

		Q_u=0.0; Q_du=0.0; Q_d2u=0.0; S_ud2u=0.0; max_v=0.0; max_dv=0.0; max_d2v=0.0;
		for(i=3;i<=n-2;i++){						//Abzug der Mittelwerte und Aufsummierung der Quadrate ohne Spikes
			if(v[i]!=v_luecke[i]){
				u[i]=v[i]-M_v;					//Geschwindigkeitsfluktuation
				Q_v=u[i]*u[i];					//Quadrate
				Q_u+=Q_v;					//aufsummiert
				if(max_v<sqrt(Q_v)) max_v=sqrt(Q_v);
				du[i]=dv[i]-M_dv;				//Steigungswert
				Q_dv=du[i]*du[i];				//Quadrate
				Q_du+=Q_dv;					//aufsummiert
				if(max_dv<sqrt(Q_dv)) max_dv=sqrt(Q_dv);
				d2u[i]=d2v[i]-M_d2v;				//Kruemmungswert
				Q_d2v=d2u[i]*d2u[i];				//Quadrate
				Q_d2u+=Q_d2v;					//aufsummiert
				if(max_d2v<sqrt(Q_d2v)) max_d2v=sqrt(Q_d2v);
				S_ud2u+=u[i]*d2u[i];

			}
		}
	
		Sigma_u=sqrt(Q_u/(n_v-5));					//Standardabweichungen der Fluktuationen, Steigungen, Kruemmungen
		Sigma_du=sqrt(Q_du/(n_dv-3));
		Sigma_d2u=sqrt(Q_d2u/(n_d2v-1));
		
		lambda_u=sqrt(2*log(n_v));					//Universelles Kriterium (normalverteilt)
		lambda_du=sqrt(2*log(n_dv));
		lambda_d2u=sqrt(2*log(n_d2v));
		
		max_u=lambda_u*Sigma_u;						//Berechnung der Halbachsen bzw. Maxi- und Minima der Grenzwertellipsen
		max_du=lambda_du*Sigma_du;
		max_d2u=lambda_d2u*Sigma_d2u;
		
		teta=atan2(S_ud2u,Q_u);						//Berechnung des Drehwinkels der Hauptachsen ueber Schwerpunkt der Punktwolke
										
		b=(max_u*tan(teta)-max_d2u)/(sin(teta)*tan(teta)-cos(teta));	//Berechnung der Halbachsen a und b der u-d2u Ellipse
		a=max_u/cos(teta)-b*tan(teta);
		
		
/********** Ermittlung der Schwellenwerte und Identifizierung von Spikes **************/
		
		
		for(i=3;i<=n-2;i++){
			u_teta=cos(teta)*u[i]-sin(teta)*d2u[i];					//Transformation zu u_teta
			d2u_teta=sin(teta)*u[i]+cos(teta)*d2u[i];				//Transformation zu d2u_teta
			Krit1=(u[i]*u[i])/(max_u*max_u)+(du[i]*du[i])/(max_du*max_du);		//Kritischer Wert nach u-du
			Krit2=(du[i]*du[i])/(max_du*max_du)+(d2u[i]*d2u[i])/(max_d2u*max_d2u);	//Kritischer Wert nach du-d2u
			Krit3=(u_teta*u_teta)/(a*a)+(d2u_teta*d2u_teta)/(b*b);			//Kritischer Wert nach u_teta-d2u_teta
			
			if(Krit1>1.0||Krit2>1.0||Krit3>1.0||corx[i]<CORR||cory[i]<CORR||corz[i]<CORR||snrx[i]<SNR||snry[i]<SNR||snrz[i]<SNR){	//Identifizierung als Ausreisser
				v_luecke[i]=v[i];
				count++;
			}
			else spike=5;		//Abbruch falls keine Ausreiser identifizierbar
		}
	spike++;
	}
	
/*********** Spline-Interpolation nach PRESS et.al., 1992 *************************/
l_nr=0;	stellen=0;
	for(i=1;i<=n;i++){				//Beginn erst, wenn alle Spike-Schleife beendet
		if(v[i]==v_luecke[i]){
			l_anfang=i;			//Festhalten der Position Lueckenanfang
			l_ende=l_anfang;
			while(v[l_ende]==v_luecke[l_ende]) l_ende++;	//Festhalten der Position Lueckenende
weite=l_ende-l_anfang;							//Bestimmung der Lueckenweite
linear=(v[l_ende+1]-v[l_anfang-1])/(weite+2);				//Lineares Steigungsmass der Luecke
if(weite<11){						//Fuer Luecken der Weite<11: Spline-Interpolation
			j=l_anfang-3; k=0;
			while(j<l_anfang && v[j]!=v_luecke[j]){
				k++;
				x[k]=(float)j;
				y[k]=(float)v[j];
				j++;
			}
			j=l_ende;
			while(j<l_ende+3 && v[j]!=v_luecke[j]){
				k++;
				x[k]=(float)j;
				y[k]=(float)v[j];
				j++;
			}
			a_1=(float)dv[l_anfang-3];
			a_n=(float)dv[l_ende+2];
			spline(x,y,k,a_1,a_n,y2);
			for(l=l_anfang;l<l_ende;l++){
				vi=splint(x,y,y2,k,l);
				v[l]=(double)vi;
			}
stellen++;					//Zaehlen der Lueckenanzahl
stopp=n;start=1;
}
else if(weite<30){					//Fuer 11<Lueckeweite<30: Lineare Interpolation, da Spline nicht stabil
for(k=0;k<=weite;k++) v[l_anfang+k]=v[l_anfang-1]+(k+1)*linear;
stellen++;
stopp=n;start=1;
}
else if((n/2)<l_anfang){		//grosse Luecke in erster Haelfte
stopp=l_anfang-1;start=1;
l_nr++;					//Anzahl
}
else{					//grosse Luecke in zweiter Haelfte
start=l_ende+1;stopp=n;
l_nr++;					//Anzahl
}
		}
	}
	
/************ Schreiben der neuen vel-Datei ********************/
t=0.02;
Krit1=(double)count/(double)n;			//Kritisches Verhaeltnis Spikes zu Gesamtmesswerten
Krit2=100.0*Krit1;
fprintf(ausgabe2,"Lamelle: %i_%i%i Punkt: %i interpoliert: %.2lf%% Luecken: %i\n",l_m,l_dm,l_cm,punkt,Krit2,l_nr);
ausgabe=fopen(punktpfad,"w");								
	if(Krit1<0.1&&l_nr<2){ 			//Loeschen der Geschwindigkeitsmesswerte, wenn mehr als 10% Spikes oder mehr als 1 große Luecke!!!
		fprintf(ausgabe,"\n");

		for(i=start;i<=stopp;i++){	//Schreiben der aufbereiteten Komponenten
			switch(komponente){
				case 1:
					vx[i]=v[i];
					break;
				case 2:
					vy[i]=v[i];
					break;
				case 3:
					vz[i]=v[i];
					break;
				}
			if(adv==1) fprintf(ausgabe,"%lf\t%lf\t%lf\t%lf\t%lf\t%lf\n",vx[i],vy[i],vz[i],corx[i],cory[i],corz[i]);
			else{ 
				fprintf(ausgabe,"%.2lf\t%i\t%i\t%lf\t%lf\t%lf\n",t,0,0,vx[i],vy[i],vz[i]);
				t=t+0.04;
			}
		}
		fclose(ausgabe);
	}
	else{
		fclose(ausgabe);
		remove(punktpfad);
	}
}
}
}
}
}
}
fclose(ausgabe2);
}
